/* -*- mode: C++; c-file-style: "bsd"; tab-width: 4 -*- */
/* jstrpilot.h - Pilot-specific definitions.
 * JStroke 1.x - Japanese Kanji handwriting recognition technology demo.
 * Copyright (C) 1997  Robert E. Wells
 * http://wellscs.com/pilot
 * mailto:robert@wellscs.com
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program (gpl.html); if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Derived from prior work by Todd David Rudick on JavaDict and StrokeDic.
 * Makes use of KANJIDIC data from Jim Breen of Monash University.
 * Further credit details available at http://wellscs.com/pilot
 * See readme.txt, ChangeLog, and gpl.html for more information.
 *
 * -------------------------------------------------------------------------*/
#ifndef __JSTRPILOT_H__
#define __JSTRPILOT_H__

// ----- Application defines -------------------------------------------------

#define dcAppCreatorId    'jStr' // Must agree with build-prc cmdline.
#define dcAppDbType       'Data' // 0x64415441 in hexadecimal
#define diAppDbCardNo      0
#define diDicDbCardNo      0
#define dcpAppDbName      "JStrokePrefDB"
#define dcpDicDbName      "JStrokeDicDB"
#define diMaxStrokes       23
#define diBigBufLen        140
#define diLitBufLen         20
#define diScreenHeight     160
#define diFontLineHeight    11	// True for font 0 with J-OS loaded...
#define diPopupXOffset      13

#define diSmoothMaxDelta     3
#define diSmoothCount        (1+diSmoothMaxDelta*2+1)

// ----- Rectangle Definition ------------------------------------------------

#define Rect_x 30
#define Rect_y 88
#define Rect_w 50  // was 60
#define Rect_h 50  // was 52
#define Rect_xt (Rect_x + Rect_w - 1)
#define Rect_yt (Rect_y + Rect_h - 1)

RectangleType Rect_bounds = { Rect_x, Rect_y, Rect_w, Rect_h };
RectangleType Rect_top = { Rect_x-2, Rect_y-2, Rect_w+4, 2 };
RectangleType Rect_bot = { Rect_x-2, Rect_y+Rect_h, Rect_w+4, 2 };
RectangleType Rect_left = { Rect_x-2, Rect_y, 2, Rect_h };
RectangleType Rect_right = { Rect_x+Rect_w, Rect_y, 2, Rect_h };

// ----- Preferences - persistent --------------------------------------------

#define dsPrefVersion 19

typedef struct {
	short m_PrefVersion;
	// Declare new preference fields here.
} Pref;

// ----- dcpAppDBName definitions --------------------------------------------
// We use the main application database to store preferences, and other
// working data either for convenience or to preserve state between runs.
// Real bulk "data" will probably be stored in separate databases.

// We put the RawStrokes memory here, as it amounts to ~14KB, and that
// is too much to put in the tmp heap, which is limited to 32KB; with
// RawStrokes in the heap, we start failing to get backing storage for 
// popup windows, and the screen decays; the app display can't come up
// for lack of memory.  Not a pretty sight.  So we store it in our Pref'
// database, locked down while we are running, and deleted as we exit to
// save space for the user.  -rwells, 970713.

#define dsDbPref 0				// DB Index for Preferences
#define dsDbText 1				// DB Index for Romanji entry field text
#define dsDbRawStrokes 2        // DB Index for RawStrokes memory

#define dsDbFirstDelete dsDbRawStrokes // Only needed while app is running...
#define dsDbMaxIndex 2			// Equal to highest index in use...


// ----- Working Context -----------------------------------------------------

typedef enum {
	eStarting = 0,
	eStroking = 1,
	eStrokeDone = 2,
} State;

typedef struct {
	State       m_eState;
	RawStroke*  m_pRawStrokes;
	UInt        m_sCurStroke;

	Boolean     m_bHasSave;
	short       m_sSaveX;
	short       m_sSaveY;

	Boolean     m_bHasRaw;
	short       m_sRawX;
	short       m_sRawY;

	DmOpenRef   m_pAppDb;
	DmOpenRef   m_pDicDb;
	FieldPtr    m_pTextField;
	ControlPtr  m_pPopupTrigger;
	ListPtr     m_pList;
	ListMem*    m_pListMem;
	CharPtr     m_cpBuf;
	CharPtr     m_cpLit;
	UInt        m_iSmoothRawX[diSmoothCount];
	UInt        m_iSmoothRawY[diSmoothCount];
	UInt        m_iSmoothIn;
	UInt        m_iSmoothOut;

	UInt        m_iRawCnt;
	UInt        m_iThinCnt;
} Context;

// ----- Function prototypes -------------------------------------------------

void      StartApplication(void);
void      StopApplication(void);

void      OpenDatabase(void);
void      CloseDatabase(void);

void      OpenStrokeDic(void);
void      CloseStrokeDic(void);

void      AppOpenForm(void);
void      AppCloseForm(void);

Err       LoadPreferences(DmOpenRef pAppDb);
void      SavePreferences(DmOpenRef pAppDb);

void      EventLoop(void);

Boolean   AppEventHandler(EventPtr event);
Boolean   AppPenEvent(EventPtr pEv);
void      AppPenSmooth(SWord ex, SWord ey, Boolean bStrokeDone);
void      AppPenSmoothOut(UInt iOutMin, UInt iOutMax);
void      AppPenSave(SWord ex, SWord ey);
Boolean   AppPenThin(SWord ex, SWord ey);
void      AppPenStore(SWord ex, SWord ey, UInt idx);
void      AppPenDraw(SWord x1, SWord y1, SWord x2, SWord y2);

void      AppPenIntercept(SWord iRawX, SWord iRawY,
								 SWord* ipX, SWord* ipY);

void      AppRecognize(void);
void      AppErase(void);
void      AppAngles(void);
void      AppReplay(void);

void      AppDrawForm(void);

ListMem*  AppEmptyList(void);
void      AppListReplace(ListMem* pListMem);

#endif /*__JSTRPILOT_H__*/
/* ----- End of jstrpilot.h ----------------------------------------------- */
